package com.jobs.lib_v1.app;

import com.jobs.lib_v1.db.DataAppCacheDB;
import com.jobs.lib_v1.db.DataAppCoreDB;
import com.jobs.lib_v1.db.DataAppDictDB;

import android.graphics.drawable.Drawable;

/**
 * 应用核心信息
 *
 *   1.获取设备号、设备操作系统版本号等。
 *   2.判断设备是否为模拟器。
 *   3.检测当前网络状态。
 */
public class AppCoreInfo {
	private static DataAppCoreDB appCoreDB = null; // 应用核心数据库
	private static DataAppCacheDB appCacheDB = null; // 缓存数据库
	private static DataAppDictDB appDictDB = null; // 存放数据字典的数据库

	/**
	 * 初始化数据库
	 *
	 * @author solomon.wen
	 * @date 2012-09-07
	 */
	public static void init(){
		getCacheDB();
		getCoreDB();
		getDictDB();
	}

	/**
	 * 获取字符串资源
	 * 
	 * @author solomon.wen
	 * @date 2012-03-27
	 * @param resourceID 字符串资源的ID
	 * @return String 字符串
	 */
	public static String getString(int resourceID){
		try {
			return AppMain.getApp().getString(resourceID);
		} catch(Exception e){
			AppUtil.print(e);
			return "";
		}
	}

	/**
	 * 获取图形资源
	 * 
	 * @author solomon.wen
	 * @date 2012-03-27
	 * @param resourceID 图形资源的ID
	 * @return String 图形
	 */
	public static Drawable getDrawable(int resourceID){
		try {
			return AppMain.getApp().getResources().getDrawable(resourceID);
		} catch (Exception e){
			AppUtil.print(e);
			return null;
		}
	}

	/**
	 * 获取全局缓存数据库对象
	 *
	 * @author solomon.wen
	 * @date 2011-12-1
	 * @return DataAppCacheDB 缓存数据库对象
	 */
	public static synchronized DataAppCacheDB getCacheDB(){
		if(null == appCacheDB){
			appCacheDB = new DataAppCacheDB();
		}

		return appCacheDB;
	}

	/**
	 * 获取全局应用设定数据库对象
	 *
	 * @author solomon.wen
	 * @date 2011-12-2
	 * @return DataAppCoreDB 应用设定数据库对象
	 */
	public static synchronized DataAppCoreDB getCoreDB(){
		if(null == appCoreDB){
			appCoreDB = new DataAppCoreDB();
		}

		return appCoreDB;
	}

	/**
	 * 获取数据字典数据库对象
	 *
	 * @author solomon.wen
	 * @date 2011-12-4
	 * @return DataAppCacheDB 数据字典数据库对象
	 */
	public static synchronized DataAppDictDB getDictDB(){
		if(null == appDictDB){
			appDictDB = new DataAppDictDB();
		}

		return appDictDB;
	}
	
	/**
	 * 销毁数据库句柄
	 *
	 * @author solomon.wen
	 * @date 2011-12-1
	 */
	public static synchronized void Destroy(){
		if(appCoreDB != null){
			appCoreDB.close();
			appCoreDB = null;
		}

		if(appCacheDB != null){
			appCacheDB.close();
			appCacheDB = null;
		}
		
		if(appDictDB != null){
			appDictDB.close();
			appDictDB = null;
		}
	}
}
