package com.jobs.lib_v1.app;

import com.jobs.lib_v1.net.NetworkManager;

import android.app.Application;

/**
 * 应用程序入口
 * 
 * @author solomon.wen
 * @date 2012-09-16
 */
public class AppMain extends Application {
	private static AppMain app = null;

	public AppMain() {
		super();
		app = this;
	}

	/**
	 * 应用初始化
	 * 
	 * @author solomon.wen
	 * @date 2012-09-16
	 */
	public void onCreate() {
		super.onCreate();

		// 检查调试开关：若未打开，则会捕获异常并发往服务器
		if (AppUtil.allowDebug()) {
			AppUtil.error(this, "当前已开启调试模式……");
		} else {
			AppException.initAppExceptionHandler();
		}

		// 初始化全局Context
		AppCoreInfo.init();

		// 监听网络变动
		NetworkManager.registerReceiver();
	}

	/**
	 * 关闭应用中所有的activity，但是应用会在后台运行
	 * 
	 * @author solomon.wen
	 * @date 2012-09-16
	 */
	public static void appFinish() {
		AppActivities.finishAllActivities();
	}

	/**
	 * 彻底退出应用
	 * 
	 * @author solomon.wen
	 * @date 2012-09-16
	 */
	public static void appExit() {
		appFinish();

		AppCoreInfo.Destroy();
		NetworkManager.unregisterReceiver();
		System.exit(0);

		app = null;
	}

	/**
	 * 返回应用程序句柄
	 * 
	 * @author solomon.wen
	 * @date 2012-09-16
	 * @return AppMain
	 */
	public static AppMain getApp() {
		return app;
	}
}
