package com.jobs.lib_v1.business;

import java.lang.reflect.Field;
import java.util.List;
import java.util.Vector;

import com.jobs.lib_v1.app.AppCoreInfo;
import com.jobs.lib_v1.app.AppUtil;
import com.jobs.lib_v1.db.DBSettings;
import com.jobs.lib_v1.db.DataAppCoreDB;
import com.jobs.lib_v1.device.DeviceUtil;
import com.jobs.lib_v1.misc.JavaReflectClass;
import com.jobs.lib_v1.misc.StringCoreUtil;

public class TRACKING {
	private static boolean isSubmittingTraceInfor = false;
	private static JavaReflectClass mTrackingPoints = new JavaReflectClass("com.jobs.settings.TRACKING_POINTS");

	/**
	 * 获取用以提交的数据跟踪信息
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return byte[]
	 */
	public static byte[] getTracePostData() {
		String userTraceData = getTraceData();

		if(null == userTraceData){
			return null;
		}

		isSubmittingTraceInfor = true;

		StringBuffer postData = new StringBuffer();
		postData.append("client_type=android&");
		postData.append("client_version=" + AppUtil.appVersionName() + "&");
		postData.append("client_os=" + "android OS " + DeviceUtil.getOSMainVersion() + "&");
		postData.append("client_uuid=" + DeviceUtil.getUUID() + "&");
		postData.append("client_data=" + userTraceData + "&");
		postData.append("client_valid=" + StringCoreUtil.md5((DeviceUtil.getUUID() + "-android" + "-" + AppUtil.appVersionName() + "-" + userTraceData).getBytes()));

		return postData.toString().getBytes();
	}

	/**
	 * 调试开关打开时，打印跟踪点名称
	 * 
	 * @author solomon.wen
	 * @date 2012-12-05
	 * @param tracePointName
	 */
	private static void record_debug(String tracePointName){
		if(!mTrackingPoints.isValid()){
			AppUtil.error("没有找到配置数据跟踪点的类： com.jobs.settings.TRACKING_POINTS");
			return;
		}

		List<Field> fieldArray = mTrackingPoints.getStaticFileds();

		for (Field field : fieldArray) {
			try {
				Object filedObject = field.get(null);
				if (filedObject instanceof String) {
					
					String curPointName = filedObject.toString();
					
					if(tracePointName.equals(curPointName)){
						AppUtil.error(AppUtil.getClassName(TRACKING.class), field.getName() + " => " + tracePointName);
						return;
					}
				}
			} catch (Exception e) {
				AppUtil.print(e);
			}
		}

		AppUtil.error(AppUtil.getClassName(TRACKING.class), "failed: " + tracePointName);
	}

	/**
	 * 跟踪记录信息
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param tracePointName 跟踪点名称
	 */
	public static void record(String tracePointName) {
		if(AppUtil.allowDebug()){
			record_debug(tracePointName);
		}

		String traceRecordSQL;

		if (isSubmittingTraceInfor) {
			traceRecordSQL = "UPDATE " + DBSettings.CORE_TABLE_USER_TRACE + " SET TRACE_COUNT_ONSUBMIT = TRACE_COUNT_ONSUBMIT + 1 WHERE TRACE_POINT = '" + tracePointName + "' ";
		} else {
			traceRecordSQL = "UPDATE " + DBSettings.CORE_TABLE_USER_TRACE + " SET TRACE_COUNT = TRACE_COUNT+1 WHERE TRACE_POINT = '" + tracePointName + "' ";
		}

		DataAppCoreDB db = AppCoreInfo.getCoreDB();
		db.begin();
		db.execute(traceRecordSQL);
		db.commit();
	}

	/***
	 * 解锁用户数据跟踪表(在提交用户数据到服务器时，该表会被锁定)
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 */
	private static void unlockTraceTable() {
		DataAppCoreDB db = AppCoreInfo.getCoreDB();

		db.begin();
		db.execute("UPDATE " + DBSettings.CORE_TABLE_USER_TRACE + " SET TRACE_COUNT = TRACE_COUNT + TRACE_COUNT_ONSUBMIT");
		db.execute("UPDATE " + DBSettings.CORE_TABLE_USER_TRACE + " SET TRACE_COUNT_ONSUBMIT = 0");
		db.commit();
	}

	/**
	 * 获取用户跟踪表中的数据
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @return String
	 */
	private static String getTraceData() {
		Vector<String[]> traceVector = AppCoreInfo.getCoreDB().query(DBSettings.CORE_TABLE_USER_TRACE, "", " TRACE_POINT ");

		if(traceVector.size() < 1){
			return null;
		}
		
		StringBuffer traceSb = new StringBuffer();

		for (int i = 0; i < traceVector.size(); i++) {
			traceSb.append(":" + traceVector.get(i)[2]);
		}

		return traceSb.toString().substring(1);
	}

	/**
	 * 数据提交完毕后执行数据表清理动作
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param removeOldTrace 是否需要删除旧的跟踪数据
	 */
	public static void cleanTrace(boolean removeOldTrace) {
		if (removeOldTrace) {
			DataAppCoreDB db = AppCoreInfo.getCoreDB();
			db.begin();
			db.execute("UPDATE " + DBSettings.CORE_TABLE_USER_TRACE + " SET TRACE_COUNT = 0");
			db.commit();
		}

		isSubmittingTraceInfor = false;
		unlockTraceTable();
	}
}
