package com.jobs.lib_v1.db;

import java.lang.reflect.Field;
import java.util.List;

import android.content.ContentValues;

import com.jobs.lib_v1.app.AppUtil;
import com.jobs.lib_v1.misc.JavaReflectClass;

/**
 * 应用核心缓存 - 数据库操作类
 */
public class DataAppCoreDB extends Data51JobDB {
	public DataAppCoreDB() {
		super(DBSettings.CORE_DB_NAME);
		initTables();
	}

	/**
	 * 初始化数据库中的表
	 * 
	 * @author solomon.wen
	 * @date 2011-12-2
	 */
	public void initTables() {
		if (!hasTable(DBSettings.CORE_TABLE_USER_TRACE)) {
			try {
				db.execSQL(DBSettings.CORE_DDL_USER_TRACE);
			} catch (Exception e) {
				AppUtil.print(e);
			}
		}

		initUserTraceTable();
	}

	/**
	 * 初始化用户跟踪表
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 */
	private void initUserTraceTable() {
		if (tableRows(DBSettings.CORE_TABLE_USER_TRACE, "") > 0) {
			return;
		}
		
		JavaReflectClass reflect = new JavaReflectClass("com.jobs.settings.TRACKING_POINTS");
		if(!reflect.isValid()){
			AppUtil.error(this, "没有找到配置数据跟踪点的类： com.jobs.settings.TRACKING_POINTS");
			return;
		}

		begin();

		List<Field> fieldArray = reflect.getStaticFileds();
		if(null != fieldArray){
			for (Field field : fieldArray) {
				try {
					Object filedObject = field.get(null);
					if (filedObject instanceof String) {
						ContentValues dbItems = new ContentValues();
	
						dbItems.put("TRACE_POINT", filedObject.toString());
						dbItems.put("TRACE_COUNT", 0);
						dbItems.put("TRACE_COUNT_ONSUBMIT", 0);
	
						insert(DBSettings.CORE_TABLE_USER_TRACE, dbItems);
					}
				} catch (Exception e) {
					AppUtil.print(e);
				}
			}
		}

		commit();
	}
}
