package com.jobs.lib_v1.list;

import java.util.HashMap;
import java.util.Map;

import android.content.res.ColorStateList;
import android.graphics.Color;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.TextView;

import com.jobs.lib_v1.data.DataItemDetail;
import com.jobs.lib_v1.device.DeviceUtil;

/**
 * DataListView 通用数据单元格
 * 
 * (2012/09/17 调整： 设置属性后返回 this， 方便后面连着设属性)
 * 
 * @author solomon.wen
 * @date 2012/09/11
 */
public class ListViewCell {
	public final static int TextViewID = ListViewCell.class.hashCode();
	private final Map<Integer, String> bindTextKeys = new HashMap<Integer, String>();
	private boolean hiddenEmptyTextView = false;
	protected int layoutID = 0;
	protected Object CELL_TAG = null;

	public ListViewCell() {
		setLayoutID(0);
		CELL_TAG = new Object();
	}

	public ListViewCell(int resID) {
		setLayoutID(resID);
	}

	public ListViewCell setLayoutID(int resID) {
		if (layoutID == resID) {
			return this;
		}

		layoutID = resID;
		CELL_TAG = new Object();

		return this;
	}

	public ListViewCell setHiddenEmptyTextView(boolean hidden) {
		hiddenEmptyTextView = hidden;
		return this;
	}

	public ListViewCell copyBindTextKeys(ListViewCell oldCell) {
		for (Integer key : oldCell.bindTextKeys.keySet()) {
			bindTextKeys.put(key, oldCell.bindTextKeys.get(key));
		}
		return this;
	}

	public ListViewCell bindTextKey(int layoutItemElementID, String itemKey) {
		if (null != itemKey) {
			bindTextKeys.put(layoutItemElementID, itemKey);
		}
		return this;
	}

	public ListViewCell bindTextKeys(int[] layoutItemElementIDList, String[] itemKeys) {
		if (null != layoutItemElementIDList && null != itemKeys && layoutItemElementIDList.length == itemKeys.length) {
			for (int i = 0; i < itemKeys.length; i++) {
				bindTextKeys.put(layoutItemElementIDList[i], itemKeys[i]);
			}
		}
		return this;
	}

	public ListViewCell clearTextKeys() {
		bindTextKeys.clear();
		return this;
	}

	public View getView(DataListAdapter adapter, int position, View cachedView) {
		View tmpView = null;

		if (null != cachedView) {
			if (cachedView.getTag() == CELL_TAG) {
				tmpView = cachedView;
			} else {
				tmpView = null;
			}
		}

		if (null == tmpView) {
			if (layoutID > 0) {
				tmpView = adapter.getCacheView(layoutID, cachedView, CELL_TAG);
			} else {
				LinearLayout rootLayout = new LinearLayout(adapter.getContext());

				ListView.LayoutParams rootParams = new ListView.LayoutParams(ListView.LayoutParams.MATCH_PARENT, ListView.LayoutParams.WRAP_CONTENT);
				rootLayout.setLayoutParams(rootParams);
				rootLayout.setGravity(Gravity.TOP);
				rootLayout.setTag(CELL_TAG);

				TextView textViewLayout = new TextView(adapter.getContext());
				ViewGroup.LayoutParams textViewParams = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
				int textViewPadding = DeviceUtil.dip2px(18);
				textViewLayout.setLayoutParams(textViewParams);
				textViewLayout.setGravity(Gravity.LEFT | Gravity.CENTER_VERTICAL);
				textViewLayout.setPadding(textViewPadding, textViewPadding, textViewPadding, textViewPadding);
				textViewLayout.setTextColor(ColorStateList.valueOf(Color.parseColor("#000000")));
				textViewLayout.setTextSize(14);
				textViewLayout.setId(TextViewID);

				rootLayout.addView(textViewLayout);

				tmpView = (View) rootLayout;
			}
		}

		if (null != bindTextKeys && bindTextKeys.size() > 0) {
			bindData(adapter, tmpView, adapter.getItem(position));
		}

		return tmpView;
	}

	private ListViewCell bindData(DataListAdapter adapter, View view, DataItemDetail item) {
		if (null == view || null == item) {
			return this;
		}

		for (Integer key : bindTextKeys.keySet()) {
			TextView textView = (TextView) view.findViewById(key);
			if (null != textView && textView instanceof TextView) {
				String str = item.getString(bindTextKeys.get(key)).trim();
				textView.setText(str);

				if (hiddenEmptyTextView) {
					if (str.length() < 1) {
						textView.setVisibility(View.GONE);
					} else {
						textView.setVisibility(View.VISIBLE);
					}
				}
			}
		}

		return this;
	}
}
