package com.jobs.lib_v1.list;

import android.content.res.ColorStateList;
import android.graphics.Color;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ListView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.jobs.lib_v1.device.DeviceUtil;
import com.jobs.lib_v1.settings.LocalStrings;

/**
 * DataListView 数据加载中的单元格
 * 
 * @author solomon.wen
 * @date 2012/09/11
 */
public class ListViewLoadingCell extends ListViewCell {
	public ListViewLoadingCell() {
		super(0);
		CELL_TAG = new Object();
	}

	@Override
	public View getView(final DataListAdapter adapter, int position, View cachedView) {
		View tmpView = null;

		if (null != cachedView) {
			if (cachedView.getTag() == CELL_TAG) {
				tmpView = cachedView;
			} else {
				tmpView = null;
			}
		}

		if (null == tmpView) {
			if (layoutID > 0) {
				tmpView = adapter.getCacheView(layoutID, cachedView, CELL_TAG);
			} else {
				LinearLayout rootLayout = new LinearLayout(adapter.getContext());

				ListView.LayoutParams rootParams = new ListView.LayoutParams(ListView.LayoutParams.MATCH_PARENT, ListView.LayoutParams.WRAP_CONTENT);
				rootLayout.setLayoutParams(rootParams);
				rootLayout.setGravity(Gravity.CENTER);
				rootLayout.setTag(CELL_TAG);

				LinearLayout parentLayout = new LinearLayout(adapter.getContext());
				ViewGroup.LayoutParams parentParams = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
				parentLayout.setLayoutParams(parentParams);
				parentLayout.setGravity(Gravity.CENTER);
				parentLayout.setOrientation(LinearLayout.HORIZONTAL);

				ProgressBar progressBar = new ProgressBar(adapter.getContext(), null, android.R.attr.progressBarStyleSmall);
				ViewGroup.LayoutParams progressParams = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
				progressBar.setLayoutParams(progressParams);

				TextView textViewLayout = new TextView(adapter.getContext());
				ViewGroup.LayoutParams textViewParams = new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
				int textViewPadding = DeviceUtil.dip2px(18);
				textViewLayout.setLayoutParams(textViewParams);
				textViewLayout.setGravity(Gravity.LEFT | Gravity.CENTER_VERTICAL);
				textViewLayout.setPadding(textViewPadding, textViewPadding, textViewPadding, textViewPadding);
				textViewLayout.setTextColor(ColorStateList.valueOf(Color.parseColor("#777777")));
				textViewLayout.setTextSize(14);
				textViewLayout.setId(TextViewID);

				parentLayout.addView(progressBar);
				parentLayout.addView(textViewLayout);
				rootLayout.addView(parentLayout);

				tmpView = (View) rootLayout;
			}
		}

		TextView textView = (TextView) tmpView.findViewById(TextViewID);
		if (null != textView) {
			textView.setText(LocalStrings.common_text_data_loading);
		}

		return tmpView;
	}
}
