package com.jobs.lib_v1.misc;

import java.io.IOException;
import java.security.spec.KeySpec;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESedeKeySpec;

import com.jobs.lib_v1.app.AppUtil;

/**
 * 通用字符串类 
 * 
 * 包含一些通用 的字符串处理和转换的静态方法
 */
public class StringCoreUtil {
	/**
	 * This character array provides the alphabet map from RFC1521.
	 */
	private final static char BASE64_ALPHABET[] = {
			// 0 1 2 3 4 5 6 7
			'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', // 0
			'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', // 1
			'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', // 2
			'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', // 3
			'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', // 4
			'o', 'p', 'q', 'r', 's', 't', 'u', 'v', // 5
			'w', 'x', 'y', 'z', '0', '1', '2', '3', // 6
			'4', '5', '6', '7', '8', '9', '+', '/' // 7
	};

	/**
	 * Decodes a 7 bit Base64 character into its binary value.
	 */
	private static int base64_ValueDecoding[] = new int[128];

	/**
	 * initializes the value decoding array from the character map
	 */
	static {
		for (int i = 0; i < base64_ValueDecoding.length; i++) {
			base64_ValueDecoding[i] = -1;
		}

		for (int i = 0; i < BASE64_ALPHABET.length; i++) {
			base64_ValueDecoding[BASE64_ALPHABET[i]] = i;
		}
	}

	/**
	 * 获取字节流的MD5报文
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param source 需要获取报文的字节流
	 * @return String MD5报文
	 */
	public static String md5(byte[] source) {
		String s = null;
		char hexDigits[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
		try {
			java.security.MessageDigest md = java.security.MessageDigest.getInstance("MD5");
			md.update(source);
			byte tmp[] = md.digest();
			char str[] = new char[16 * 2];
			int k = 0;
			for (int i = 0; i < 16; i++) {
				byte byte0 = tmp[i];
				str[k++] = hexDigits[byte0 >>> 4 & 0xf];
				str[k++] = hexDigits[byte0 & 0xf];
			}
			s = new String(str);
		} catch (Exception e) {
			AppUtil.print(e);
		}

		return s;
	}

	/**
	 * 字符串转为整数，并忽略错误
	 * 
	 * @author solomon.wen
	 * @date 2011-11-25
	 * @param str 需要转为整数的字符串
	 * @return int 转换后的整数；如出错则返回0
	 */
	public static int strToInt(String str) {
		if (str == null || str == "") {
			return 0;
		}

		int retValue;

		try {
			retValue = Integer.parseInt(str);
		} catch (NumberFormatException e) {
			AppUtil.print(e);
			retValue = 0;
		}

		return retValue;
	}

	/**
	 * 用正则表达式替换
	 * 
	 * @author solomon.wen
	 * @date 2012-12-08
	 * 
	 * @param source_string
	 * @param pattern_string
	 * @param replace_string
	 * @return String
	 */
	public static String PatternReplace(String source_string, String pattern_string, String replace_string) {
		StringBuffer sb = new StringBuffer();

		try {
			Matcher matcher = Pattern.compile(pattern_string).matcher(source_string);

			while (matcher.find()) {
				matcher.appendReplacement(sb, replace_string);
			}

			matcher.appendTail(sb);
		} catch (Exception e) {
			return "";
		}

		return sb.toString();
	}

	/**
	 * Converts a String string to a Base64 encoded string.
	 * 
	 * @param data
	 *            String
	 * @return encoded string
	 */
	public static String base64Encode(String data) {
		byte[] dataBytes = data.getBytes();
		return base64Encode(dataBytes, 0, dataBytes.length);
	}

	/**
	 * Converts a Base64 encoded string to a byte array.
	 * 
	 * @param encoded
	 *            Base64 encoded data
	 * @return decode binary data; 3 bytes for every 4 chars - minus padding
	 */
	public static byte[] base64Decode(String encoded) {
		try {
			return base64Decode(encoded, 0, encoded.length());
		} catch (IOException e) {
			AppUtil.print(e);
		}

		return "".getBytes();
	}

	/**
	 * Converts a byte array into a Base64 encoded string.
	 * 
	 * @param data
	 *            bytes to encode
	 * @param offset
	 *            which byte to start at
	 * @param length
	 *            how many bytes to encode; padding will be added if needed
	 * @return base64 encoding of data; 4 chars for every 3 bytes
	 */
	public static String base64Encode(byte[] data, int offset, int length) {
		int i;
		int encodedLen;
		char[] encoded;

		// 4 chars for 3 bytes, run input up to a multiple of 3
		encodedLen = (length + 2) / 3 * 4;
		encoded = new char[encodedLen];

		for (i = 0, encodedLen = 0; encodedLen < encoded.length; i += 3, encodedLen += 4) {
			base64EncodeQuantum(data, offset + i, length - i, encoded, encodedLen);
		}

		return new String(encoded);
	}

	/**
	 * Converts an embedded Base64 encoded string to a byte array.
	 * 
	 * @param encoded a String with Base64 data embedded in it
	 * @param offset which char of the String to start at
	 * @param length how many chars to decode; must be a multiple of 4
	 * @return decode binary data; 3 bytes for every 4 chars - minus padding
	 * @exception IOException is thrown, if an I/O error occurs reading the data
	 */
	public static byte[] base64Decode(String encoded, int offset, int length) throws IOException {
		int i;
		int decodedLen;
		byte[] decoded;

		// the input must be a multiple of 4
		if (length % 4 != 0) {
			throw new IOException("Base64 string length is not multiple of 4");
		}

		// 4 chars for 3 bytes, but there may have been pad bytes
		decodedLen = length / 4 * 3;
		if (encoded.charAt(offset + length - 1) == '=') {
			decodedLen--;
			if (encoded.charAt(offset + length - 2) == '=') {
				decodedLen--;
			}
		}

		decoded = new byte[decodedLen];
		for (i = 0, decodedLen = 0; i < length; i += 4, decodedLen += 3) {
			base64DecodeQuantum(encoded.charAt(offset + i), encoded.charAt(offset + i + 1), encoded.charAt(offset + i + 2), encoded.charAt(offset + i + 3), decoded, decodedLen);
		}

		return decoded;
	}

	/**
	 * Encodes 1, 2, or 3 bytes of data as 4 Base64 chars.
	 * 
	 * @param in
	 *            buffer of bytes to encode
	 * @param inOffset
	 *            where the first byte to encode is
	 * @param len
	 *            how many bytes to encode
	 * @param out
	 *            buffer to put the output in
	 * @param outOffset
	 *            where in the output buffer to put the chars
	 */
	private static void base64EncodeQuantum(byte in[], int inOffset, int len, char out[], int outOffset) {
		byte a = 0, b = 0, c = 0;

		a = in[inOffset];
		out[outOffset] = BASE64_ALPHABET[(a >>> 2) & 0x3F];

		if (len > 2) {
			b = in[inOffset + 1];
			c = in[inOffset + 2];
			out[outOffset + 1] = BASE64_ALPHABET[((a << 4) & 0x30) + ((b >>> 4) & 0xf)];
			out[outOffset + 2] = BASE64_ALPHABET[((b << 2) & 0x3c) + ((c >>> 6) & 0x3)];
			out[outOffset + 3] = BASE64_ALPHABET[c & 0x3F];
		} else if (len > 1) {
			b = in[inOffset + 1];
			out[outOffset + 1] = BASE64_ALPHABET[((a << 4) & 0x30) + ((b >>> 4) & 0xf)];
			out[outOffset + 2] = BASE64_ALPHABET[((b << 2) & 0x3c) + ((c >>> 6) & 0x3)];
			out[outOffset + 3] = '=';
		} else {
			out[outOffset + 1] = BASE64_ALPHABET[((a << 4) & 0x30) + ((b >>> 4) & 0xf)];
			out[outOffset + 2] = '=';
			out[outOffset + 3] = '=';
		}
	}

	/**
	 * Decode 4 Base64 chars as 1, 2, or 3 bytes of data.
	 * 
	 * @param in1
	 *            first char of quantum to decode
	 * @param in2
	 *            second char of quantum to decode
	 * @param in3
	 *            third char of quantum to decode
	 * @param in4
	 *            forth char of quantum to decode
	 * @param out
	 *            buffer to put the output in
	 * @param outOffset
	 *            where in the output buffer to put the bytes
	 */
	private static void base64DecodeQuantum(char in1, char in2, char in3, char in4, byte[] out, int outOffset) throws IOException {
		int a = 0, b = 0, c = 0, d = 0;
		int pad = 0;

		a = base64_ValueDecoding[in1 & 127];
		b = base64_ValueDecoding[in2 & 127];

		if (in4 == '=') {
			pad++;
			if (in3 == '=') {
				pad++;
			} else {
				c = base64_ValueDecoding[in3 & 127];
			}
		} else {
			c = base64_ValueDecoding[in3 & 127];
			d = base64_ValueDecoding[in4 & 127];
		}

		if (a < 0 || b < 0 || c < 0 || d < 0) {
			throw new IOException("Invalid character in Base64 string");
		}

		// the first byte is the 6 bits of a and 2 bits of b
		out[outOffset] = (byte) (((a << 2) & 0xfc) | ((b >>> 4) & 3));

		if (pad < 2) {
			// the second byte is 4 bits of b and 4 bits of c
			out[outOffset + 1] = (byte) (((b << 4) & 0xf0) | ((c >>> 2) & 0xf));

			if (pad < 1) {
				// the third byte is 2 bits of c and 4 bits of d
				out[outOffset + 2] = (byte) (((c << 6) & 0xc0) | (d & 0x3f));
			}
		}
	}

	/**
	 * 从字符串转换为时间
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @param strDate
	 * @return Date
	 */
	public static Date StringToDate(String strDate){
		return StringToDate(strDate, null, new Date());
	}

	/**
	 * 从字符串转换为时间
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @param strDate
	 * @param defaultDate
	 * @return Date
	 */
	public static Date StringToDate(String strDate, Date defaultDate){
		return StringToDate(strDate, null, defaultDate);
	}

	/**
	 * 从字符串转换为时间
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @param strDate
	 * @param strFormatter
	 * @return Date
	 */
	public static Date StringToDate(String strDate, String strFormatter, Date defaultDate){
		if(null == strFormatter){
			strFormatter = "";
		}

		strFormatter = strFormatter.trim();

		if(strFormatter.length() < 1){
			strFormatter = "yyyy-MM-dd HH:mm:ss";
		}
		
		if(null == strDate){
			return defaultDate;
		}

		SimpleDateFormat sf = null;

		Date retDate = defaultDate;

		// 默认识别“年-月-日  时:分:秒”
		try {
			sf = new SimpleDateFormat(strFormatter, Locale.US);
			retDate = sf.parse(strDate);
			return retDate;
		} catch (Exception e) {
			AppUtil.print(e);
		}

		// 识别  “年-月-日”
		try {
			sf = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
			retDate = sf.parse(strDate);
			return retDate;
		} catch (Exception e) {
			AppUtil.print(e);
		}

		// 识别“年-月”
		try {
			sf = new SimpleDateFormat("yyyy-MM", Locale.US);
			retDate = sf.parse(strDate);
			return retDate;
		} catch (Exception e) {
			AppUtil.print(e);
		}

		return defaultDate;
	}

	/**
	 * 当前时间日期对象格式化为字符串
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @return String
	 */
	public static String DateToString(){
		return DateToString(null, null);
	}

	/**
	 * 时间日期对象格式化为字符串
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @param inputDate
	 * @return String
	 */
	public static String DateToString(Date inputDate){
		return DateToString(inputDate, null);
	}

	/**
	 * 时间日期对象格式化为字符串
	 * 
	 * @author solomon.wen
	 * @date 2012-04-01
	 * @param inputDate
	 * @param strFormatter
	 * @return String
	 */
	public static String DateToString(Date inputDate, String strFormatter){
		if(null == inputDate){
			inputDate = new Date();
		}

		if(null == strFormatter){
			strFormatter = "";
		}

		strFormatter = strFormatter.trim();

		if(strFormatter.length() < 1){
			strFormatter = "yyyy-MM-dd HH:mm:ss";
		}

		SimpleDateFormat sf =new SimpleDateFormat(strFormatter, Locale.US);
		String retStr = sf.format(inputDate);

		return retStr;
	}

	/**
	 * 字节转换为十六进制字符串
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param data
	 * @return String
	 */
	public static String byte2hex(byte[] data) {
		if(null == data){
			return null;
		}

		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < data.length; i++) {
			sb.append(String.format("%02x", data[i]));
		}

		return sb.toString();
	}

	/**
	 * 十六进制字符串转换为字节
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param str
	 * @return byte[]
	 */
	public static byte[] hexToBytes(String str) {
		if (str == null) {
			return null;
		} else if (str.length() < 2 || str.length() % 2 != 0) {
			return null;
		} else {
			int len = str.length() / 2;
			byte[] buffer = new byte[len];

			for (int i = 0; i < len; i++) {
				try{
					buffer[i] = (byte) Integer.parseInt(str.substring(i * 2, i * 2 + 2), 16);
				} catch(NumberFormatException e){
					AppUtil.print(e);
					return null;
				}
			}

			return buffer;
		}
	}

	/**
	 * 对字符串采用三重DES加密，然后进行Base64编码
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param data
	 * @param key
	 * @return String
	 */
	public static String base64_encrypt_3des(String data, String key){
		if(null == data || null == key || data.length() < 1 || key.length() < 1){
			return null;
		}

		try {
			byte[] byte_data = data.getBytes();
			byte[] byte_key = key.getBytes();
			byte[] byte_res = encrypt_3des(byte_data, byte_key);

			return base64Encode(byte_res, 0, byte_res.length);
		} catch (Exception e) {
			AppUtil.print(e);
			return null;
		}
	}

	/**
	 * 对字符串进行Base64解码，然后再进行3DES解密
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param data
	 * @param key
	 * @return String
	 */
	public static String base64_decrypt_3des(String data, String key){
		if(null == data || null == key || data.length() < 1 || key.length() < 1){
			return null;
		}
		
		try {
			byte[] byte_data = base64Decode(data);

			if(null == byte_data || byte_data.length < 1){
				return null;
			}

			byte[] byte_key = key.getBytes();
			byte[] byte_res = decrypt_3des(byte_data, byte_key);

			if(null == byte_res){
				return null;
			}
			
			int pos = 0;
			while(pos < byte_res.length && byte_res[pos] != 0){
				pos++;
			}
			
			if(pos < 1){
				return "";
			}

			byte[] byte_ret = new byte[pos];
			System.arraycopy(byte_res, 0, byte_ret, 0, pos);

			return new String(byte_ret);
		} catch (Exception e) {
			AppUtil.print(e);
			return null;
		}
	}

	/**
	 * 对字节数据采用三重DES加密
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param data
	 * @param key
	 * @return byte[]
	 */
	public static byte[] encrypt_3des(byte data[], byte key[]) {
		if(null == data || null == key || data.length < 1 || key.length < 1){
			return null;
		}

		try {
			byte[] k = new byte[24];

			int len = data.length;
			if (data.length % 8 != 0) {
				len = data.length - data.length % 8 + 8;
			}
			byte[] needData = null;
			if (len != 0)
				needData = new byte[len];

			for (int i = 0; i < len; i++) {
				needData[i] = 0x00;
			}

			System.arraycopy(data, 0, needData, 0, data.length);
			
			for (int i = 0; i < k.length; i++) {
				k[i] = 0x00;
			}

			if (key.length < 24) {
				System.arraycopy(key, 0, k, 0, key.length);
			} else {
				System.arraycopy(key, 0, k, 0, 24);
			}

			KeySpec ks = new DESedeKeySpec(k);
			SecretKeyFactory kf = SecretKeyFactory.getInstance("DESede");
			SecretKey ky = kf.generateSecret(ks);

			Cipher c = Cipher.getInstance("DESede/ECB/NoPadding");
			c.init(Cipher.ENCRYPT_MODE, ky);
			return c.doFinal(needData);
		} catch (Exception e) {
			AppUtil.print(e);
			return null;
		}
	}

	/**
	 * 三重DES解密字节数据
	 * 
	 * @author solomon.wen
	 * @date 2012-08-01
	 * @param data
	 * @param key
	 * @return byte[]
	 */
	public static byte[] decrypt_3des(byte data[], byte key[]) {
		if(null == data || null == key || data.length < 1 || key.length < 1){
			return null;
		}

		try {
			byte[] k = new byte[24];

			int len = data.length;
			if (data.length % 8 != 0) {
				len = data.length - data.length % 8 + 8;
			}
			byte[] needData = null;
			if (len != 0)
				needData = new byte[len];

			for (int i = 0; i < len; i++) {
				needData[i] = 0x00;
			}

			System.arraycopy(data, 0, needData, 0, data.length);

			for (int i = 0; i < k.length; i++) {
				k[i] = 0x00;
			}

			if (key.length < 24) {
				System.arraycopy(key, 0, k, 0, key.length);
			} else {
				System.arraycopy(key, 0, k, 0, 24);
			}

			KeySpec ks = new DESedeKeySpec(k);
			SecretKeyFactory kf = SecretKeyFactory.getInstance("DESede");
			SecretKey ky = kf.generateSecret(ks);

			Cipher c = Cipher.getInstance("DESede/ECB/NoPadding");
			c.init(Cipher.DECRYPT_MODE, ky);
			return c.doFinal(needData);
		} catch (Exception e) {
			AppUtil.print(e);
			return null;
		}
	}
}
